@@ -19,6 +19,12 @@ type solver
     for universe consistency (cf. Cudf_checker.is_consistent) *)
 val load : ?check : bool -> Cudf.universe -> solver
 
+(** Turn a result from Diagnostic_int into one of Diagnostic *)
+val result : Depsolver_int.identity -> Cudf.universe -> Diagnostic_int.result -> Diagnostic.result
+
+(** Turn a request from Diagnostic_int into one of Diagnostic *)
+val request : Cudf.universe -> Diagnostic_int.request -> Diagnostic.request
+
 (** check if the given package can be installed in the universe 
  
     @param global_constraints : enforce global constraints on the given
@@ -50,6 +56,14 @@ val find_broken : ?global_constraints:bool -> Cudf.universe -> Cudf.package list
 (** return the list of installable packages *)
 val find_installable : ?global_constraints:bool -> Cudf.universe -> Cudf.package list
 
+(** return the list of broken packages *)
+val find_listbroken : ?global_constraints:bool -> Cudf.universe ->
+  Cudf.package list -> Cudf.package list
+
+(** return the list of installable packages *)
+val find_listinstallable : ?global_constraints:bool -> Cudf.universe ->
+  Cudf.package list -> Cudf.package list
+
 (** [univcheck ] check if all packages in the universe can be installed.
     Since not all packages are directly tested for installation, if a packages
     is installable, the installation might be empty. To obtain an installation
@@ -119,4 +133,5 @@ type solver_result =
     if ?explain is specified and there is no solution for the give request, the
     result will contain the failure reason.
 *)
-val check_request : ?cmd : string -> ?criteria : string -> ?explain : bool -> Cudf.cudf -> solver_result
+val check_request : ?cmd : string -> ?callback:(int array * Diagnostic.diagnosis -> unit) -> 
+  ?criteria:string -> ?explain : bool -> Cudf.cudf -> solver_result
