@@ -21,12 +21,13 @@ include Util.Logging(struct let label = "apt-cudf backend" end) ;;
 
 module Options = struct
   open OptParse
-  let description = "apt-get backend (EDSP v. 0.4)"
+  let description = "apt-get backend (EDSP > 0.4)"
   let options = OptParser.make ~description
   include Boilerplate.MakeOptions(struct let options = options end)
 
   let dump = StdOpt.store_true ()
   let noop = StdOpt.store_true ()
+  let human = StdOpt.store_true ()
   let solver = StdOpt.str_option ()
   let criteria = StdOpt.str_option ()
   let explain = StdOpt.store_true ()
@@ -37,11 +38,11 @@ module Options = struct
   open OptParser
   add options ~long_name:"conf" ~help:"configuration file (default:/etc/apt-cudf.conf)" conffile;
   add options ~long_name:"dump" ~help:"dump the cudf universe and solution" dump;
-  add options ~long_name:"noop" ~help:"Do nothing, implies --dump" noop;
+  add options ~long_name:"noop" ~help:"Do nothing" noop;
   add options ~short_name:'s' ~long_name:"solver" ~help:"external solver" solver;
   add options ~short_name:'c' ~long_name:"criteria" ~help:"optimization criteria" criteria;
-  add options ~short_name:'e' ~long_name:"explain" ~help:"summary" explain;
-
+  add options ~short_name:'e' ~long_name:"explain" ~help:"print installation summary" explain;
+  add options ~long_name:"human" ~help:"print human readable installation errors" human;
   add options ~long_name:"native-arch" ~help:"Native architecture" native_arch;
   add options ~long_name:"foreign-archs" ~help:"Foreign architectures" foreign_archs;
 
@@ -49,10 +50,14 @@ end
 
 let fatal fmt =
   Printf.kprintf (fun s ->
-    Format.printf "Error: %s (%s)@." 
-      (OptParse.Opt.get Options.solver) (OptParse.Opt.get Options.criteria) ;
+    if OptParse.Opt.is_set Options.solver then begin
+      Format.printf "Error: %s" (OptParse.Opt.get Options.solver);
+      if OptParse.Opt.is_set Options.criteria then
+        Format.printf " \"%s\"" (OptParse.Opt.get Options.criteria);
+      Format.printf "@."
+    end;
     Format.printf "Message: %s@." s;
-    exit 0
+    exit 1
   ) fmt
 ;;
 
@@ -82,7 +87,7 @@ let pp_pkg fmt (s,univ) =
   try
     let p = CudfAdd.Cudf_set.choose s in
     let pkg = Hashtbl.find univ (p.Cudf.package,p.Cudf.version) in
-    let apt_id = Debian.Packages.assoc "apt-id" pkg.Packages.extras in
+    let apt_id = Debian.Packages.assoc "APT-ID" pkg.Packages.extras in
     Format.fprintf fmt "%s\n" apt_id;
     Format.fprintf fmt "Package: %s\n" pkg.Packages.name;
     Format.fprintf fmt "Version: %s\n" pkg.Packages.version;
@@ -163,12 +168,12 @@ let choose_criteria ?(criteria=None) ~conffile solver request =
     with Not_found ->
       try List.assoc "*" conf 
       with Not_found -> [
-        ("install", "-removed,-changed");
-        ("remove", "-removed,-changed");
-        ("upgrade","-new,-removed,-notuptodate");
-        ("dist-upgrade","-notuptodate,-new,-removed");
-        ("trendy","-removed,-notuptodate,-unsat_recommends,-new");
-        ("paranoid","-removed,-changed")
+        "upgrade", "-count(new),-count(removed),-notuptodate(solution)";
+        "dist-upgrade", "-notuptodate(solution),-count(new)";
+        "install", "-count(removed),-count(changed)";
+        "remove", "-count(removed),-count(changed)";
+        "trendy", "-count(removed),-notuptodate(solution),-unsat_recommends(solution),-count(new)";
+        "paranoid", "-count(removed),-count(changed)";
       ]
   in
   match criteria,request.Edsp.preferences with
@@ -184,21 +189,24 @@ let choose_criteria ?(criteria=None) ~conffile solver request =
 let parse_solver_spec filename =
   let (exec, version) = (ref "", ref "") in
   begin try
-    let ic = open_in filename in
-    while true do
-      let l = input_line ic in
-      if String.starts_with l "exec: " then
-        exec := String.strip (snd (String.split l " "))
-      else if String.starts_with l "cudf-version: " then
-        Scanf.sscanf l "cudf-version: %s " (fun s -> version := s);
-    done;
-    close_in ic
-    with
-      | Sys_error _ -> fatal "cannot parse CUDF solver specification %s" filename
-      | End_of_file -> ()
-      | Scanf.Scan_failure err ->
-        fatal "parse error while reading CUDF solver specification %s: %s"
-	  filename err
+    if Sys.file_exists filename then begin
+      let ic = open_in filename in
+      while true do
+        let l = input_line ic in
+        if String.starts_with l "exec: " then
+          exec := String.strip (snd (String.split l " "))
+        else if String.starts_with l "cudf-version: " then
+          Scanf.sscanf l "cudf-version: %s " (fun s -> version := s);
+      done;
+      close_in ic
+    end else
+      fatal "Solver %s not found" filename
+  with
+    | Sys_error _ -> fatal "cannot parse CUDF solver specification %s" filename
+    | End_of_file -> ()
+    | Scanf.Scan_failure err ->
+      fatal "parse error while reading CUDF solver specification %s: %s"
+        filename err
   end;
   if !exec = "" || !version = "" then
     fatal "incomplete CUDF solver specification %s" filename;
@@ -210,33 +218,10 @@ let parse_solver_spec filename =
   (!exec,!version)
 ;;
 
-let get_architectures native_opt foreign =
-  let cmd = "apt-config dump" in
-  let arch = ref "" in
-  let archs = ref [] in
-  let aux () =
-    let out = Std.input_list (Unix.open_process_in cmd) in
-    List.iter (fun s ->
-      let key, value =  ExtString.String.split s " " in
-      if key = "APT::Architecture" then
-        arch := ExtString.String.slice ~first: 1 ~last:(-2) value
-      else if key = "APT::Architectures::" || key = "APT::Architectures" then
-        let s = ExtString.String.slice ~first:1 ~last:(-2) value in
-        if s <> "" then
-          archs := (ExtString.String.slice ~first:1 ~last:(-2) value)::!archs
-    ) out;
-    debug "Atomatically set native as %s and foreign archs as %s" !arch (String.concat "," !archs);
-  in
-  match native_opt, foreign with 
-  |None,None     -> aux () ; (!arch,List.filter ((<>) !arch) !archs)
-  |None,Some l   -> fatal "Native arch is missing while Foregin archs are specified"
-  |Some a,None   -> (a,[])
-  |Some a,Some l -> (a,l)
-;;
-
 let main () =
   let timer1 = Util.Timer.create "parsing" in
   let timer2 = Util.Timer.create "conversion" in
+  let timer3 = Util.Timer.create "solution" in
   let args = OptParse.OptParser.parse_argv Options.options in
   Boilerplate.enable_debug (OptParse.Opt.get Options.verbose);
   Boilerplate.enable_bars (OptParse.Opt.get Options.progress) [] ;
@@ -248,20 +233,6 @@ let main () =
   debug "CUDFSOLVERS=%s" solver_dir;
   (* debug "TMPDIR=%s" waiting for ocaml 4.0 *)
 
-  let (native_arch,foreign_archs) = 
-    get_architectures 
-      (OptParse.Opt.opt Options.native_arch) 
-      (OptParse.Opt.opt Options.foreign_archs)
-  in
-
-  let solver = 
-    if OptParse.Opt.is_set Options.solver then
-      OptParse.Opt.get Options.solver
-    else 
-      Filename.basename(Sys.argv.(0))
-  in
-  let exec_pat = fst (parse_solver_spec (Filename.concat solver_dir solver)) in
-
   let ch = 
     match args with 
     |[] -> (IO.input_channel stdin)
@@ -269,9 +240,16 @@ let main () =
   in
   
   Util.Timer.start timer1;
-  (* archs are inferred by calling apt-config dump *)
-  let archs = native_arch::foreign_archs in
-  let (request,pkglist) = Edsp.input_raw_ch ~archs ch in
+  let (request,pkglist) = Edsp.input_raw_ch ch in
+
+  let (native_arch,foreign_archs) = 
+    Boilerplate.get_architectures
+      request.Edsp.architecture
+      request.Edsp.architectures
+      (OptParse.Opt.opt Options.native_arch) 
+      (OptParse.Opt.opt Options.foreign_archs)
+  in
+
   let request =
     match apt_get_cmdline with
     |"" -> request
@@ -292,12 +270,12 @@ let main () =
     CudfAdd.add_properties Debcudf.preamble l
   in
 
-  let univ = Hashtbl.create (2*(List.length pkglist)-1) in
   let options = {
     Debcudf.default_options with 
     Debcudf.native = native_arch;
     Debcudf.foreign = foreign_archs }
   in 
+  let univ = Hashtbl.create (2*(List.length pkglist)-1) in
   let cudfpkglist = 
     List.filter_map (fun pkg ->
       let p = Edsp.tocudf tables ~options pkg in
@@ -311,28 +289,46 @@ let main () =
       end
     ) pkglist 
   in
-
-  let cudfdump = Filename.temp_file "apt-cudf-universe" ".cudf" in
-  if OptParse.Opt.get Options.dump || OptParse.Opt.get Options.noop then begin
-    Printf.printf "Apt-cudf: dump cudf universe in %s\n" cudfdump;
-    let oc = open_out cudfdump in
-    Cudf_printer.pp_preamble oc default_preamble;
-    Printf.fprintf oc "\n";
-    Cudf_printer.pp_packages oc cudfpkglist;
-    close_out oc
-  end;
-
   let universe = 
     try Cudf.load_universe cudfpkglist
     with Cudf.Constraint_violation s ->
       fatal "(CUDF) Malformed universe %s" s
   in
+
+(*
+  let universe = 
+    let initialsize = (2 * (List.length pkglist) - 1 ) in
+    let univcache = Cudf.empty_universe ~size:initialsize () in
+    List.iter (fun pkg ->
+      let p = Edsp.tocudf tables ~options pkg in
+      Hashtbl.add univ (p.Cudf.package,p.Cudf.version) pkg;
+      try Cudf.add_package univcache p
+      with Cudf.Constraint_violation s ->
+(*        warning "Possibly duplicated package (same version, name and architecture) : (%s,%s,%s)"
+          pkg.Packages.name pkg.Packages.version pkg.Packages.architecture;
+          *)
+        fatal "(CUDF) Malformed universe %s" s
+    ) pkglist ;
+    univcache
+  in
+  *)
+
+  let cudfdump = Filename.temp_file "apt-cudf-universe" ".cudf" in
+  if OptParse.Opt.get Options.dump then begin
+    info "Dump cudf universe in %s\n" cudfdump;
+    let oc = open_out cudfdump in
+    Cudf_printer.pp_preamble oc default_preamble;
+    Printf.fprintf oc "\n";
+    Cudf_printer.pp_universe oc universe;
+    close_out oc
+  end;
+
   let cudf_request = Edsp.requesttocudf tables universe request in
   let cudf = (default_preamble,universe,cudf_request) in
   Util.Timer.stop timer2 ();
 
-  if OptParse.Opt.get Options.dump || OptParse.Opt.get Options.noop then begin
-    Printf.printf "Apt-cudf: append cudf request to %s\n" cudfdump;
+  if OptParse.Opt.get Options.dump then begin
+    info "Append cudf request to %s\n" cudfdump;
     let oc = open_out_gen 
       [Open_wronly; Open_append; Open_creat; Open_text]
       0o666 cudfdump 
@@ -342,8 +338,18 @@ let main () =
     close_out oc
   end;
 
-  (* do nothing, we exit here after dumping the universe *)
-  if OptParse.Opt.get Options.noop then exit(0);
+  (* do nothing. *)
+  if OptParse.Opt.get Options.noop then begin
+    info "Noop & Exit."; exit(0)
+  end;
+
+  let solver = 
+    if OptParse.Opt.is_set Options.solver then
+      OptParse.Opt.get Options.solver
+    else 
+      Filename.basename(Sys.argv.(0))
+  in
+  let exec_pat = fst (parse_solver_spec (Filename.concat solver_dir solver)) in
 
   let cmdline_criteria = OptParse.Opt.opt Options.criteria in
   let conffile = OptParse.Opt.get Options.conffile in
@@ -351,16 +357,24 @@ let main () =
   OptParse.Opt.set Options.criteria criteria ;
 
   let solpre,soluniv = 
-    let explain = OptParse.Opt.get Options.explain in
+    let from_cudf (p,v) = (p,Debian.Debcudf.get_real_version tables (p,v)) in
+    let pp = CudfAdd.pp from_cudf in
+    let explain = OptParse.Opt.get Options.human in
     match Algo.Depsolver.check_request ~cmd:exec_pat ~criteria ~explain cudf with
     |Algo.Depsolver.Error s -> fatal "%s" s
-    |Algo.Depsolver.Unsat _ -> fatal "(UNSAT) No Solutions according to the give preferences"
+    |Algo.Depsolver.Unsat None ->
+      fatal "(UNSAT) No Solutions according to the given preferences"
+    |Algo.Depsolver.Unsat Some d -> begin
+      Format.printf "Error: (UNSAT) No Solutions according to the given preferences@.";
+      Format.printf "%a@." (Algo.Diagnostic.fprintf_human ~prefix:"Message: " ~pp) d;
+      exit 1
+    end
     |Algo.Depsolver.Sat s -> s
   in
 
   if OptParse.Opt.get Options.dump then begin
     let cudfsol = Filename.temp_file "apt-cudf-solution" ".cudf" in
-    Printf.printf "Apt-cudf: dump cudf solution in %s\n" cudfsol;
+    info "Dump cudf solution in %s\n" cudfsol;
     let oc = open_out cudfsol in
     Cudf_printer.pp_preamble oc default_preamble;
     Printf.fprintf oc "\n";
@@ -368,23 +382,22 @@ let main () =
     close_out oc
   end;
 
+  Util.Timer.start timer3;
   let diff = CudfDiff.diff universe soluniv in
   let empty = ref true in
+  let cache = CudfAdd.Cudf_hashtbl.create 1023 in
   Hashtbl.iter (fun pkgname s ->
     let inst = s.CudfDiff.installed in
     let rem = s.CudfDiff.removed in
     match CudfAdd.Cudf_set.is_empty inst, CudfAdd.Cudf_set.is_empty rem with
-    |false,true -> begin
-        empty := false;
-        Format.printf "Install: %a@." pp_pkg (inst,univ)
+    |false,_ -> begin
+      empty := false;
+      List.iter (fun pkg -> CudfAdd.Cudf_hashtbl.add cache pkg ()) (CudfAdd.Cudf_set.elements inst);
+      Format.printf "Install: %a@." pp_pkg (inst,univ)
     end
     |true,false -> begin
-        empty := false;
-        Format.printf "Remove: %a@." pp_pkg (rem,univ)
-    end
-    |false,false -> begin
-        empty := false;
-        Format.printf "Install: %a@." pp_pkg (inst,univ)
+      empty := false;
+      Format.printf "Remove: %a@." pp_pkg (rem,univ)
     end
     |true,true -> ()
   ) diff;
@@ -397,11 +410,16 @@ let main () =
       if (CudfAdd.Cudf_set.is_empty s.CudfDiff.installed) then begin
         List.iter (fun pkg -> 
           empty := false;
-          Format.printf "Install: %a@." pp_pkg ((CudfAdd.Cudf_set.singleton pkg),univ)
+          if CudfAdd.Cudf_hashtbl.mem cache pkg then () 
+          else begin
+            CudfAdd.Cudf_hashtbl.add cache pkg ();
+            Format.printf "Install: %a@." pp_pkg ((CudfAdd.Cudf_set.singleton pkg),univ);
+          end
         ) (CudfAdd.who_provides soluniv (n,c))
       end
     with Not_found -> ()
   ) cudf_request.Cudf.install;
+  Util.Timer.stop timer3 ();
 
   if OptParse.Opt.get Options.explain then begin
     let (i,u,d,r) = CudfDiff.summary universe diff in
