@@ -97,7 +97,7 @@ let main () =
   let posargs = OptParse.OptParser.parse_argv Options.options in
   Boilerplate.enable_debug (OptParse.Opt.get Options.verbose);
   let universe = 
-    let (_,u,_,_) = Boilerplate.load_universe posargs in
+    let (_,u,_,_,_) = Boilerplate.load_universe posargs in
     if OptParse.Opt.get Options.trim then
       Depsolver.trim ~global_constraints:false u
     else u
