@@ -10,8 +10,9 @@
 (*  library, see the COPYING file for more information.                               *)
 (**************************************************************************************)
 
-(** Representation of a apt-get <-> solvers protocol edsp 0.3 *)
+(** Representation of a apt-get <-> solvers protocol edsp > 0.4 *)
 
+module Pcre = Re_pcre
 open ExtLib
 open Common
 
@@ -21,31 +22,37 @@ type request = {
   request : string;
   install : Format822.vpkg list;
   remove : Format822.vpkg list;
+  architecture : Format822.architecture option;
+  architectures : Format822.architectures;
   autoremove : bool;
   upgrade : bool;
   distupgrade : bool;
   strict_pin : bool;
   preferences: string;
+  cmdline: string;
 }
 
 let default_request = {
   request = "";
   install = [];
   remove = [];
+  architecture = None;
+  architectures = [];
   autoremove = false;
   upgrade = false;
   distupgrade = false;
   strict_pin = false;
-  preferences = ""
+  preferences = "";
+  cmdline = ""
 }
 
 (* convert a apt command line request to edsp request *)
 let from_apt_request arch request = function
   |Apt.Install vpkgreqlist ->
       List.fold_left (fun acc -> function
-        |(Some Format822.I, ((n,None),c), _) -> {acc with install = ((n,Some arch),c) :: acc.install}
-        |(Some Format822.R, ((n,None),c), _) -> {acc with remove =  ((n,Some arch),c) :: acc.remove}
-        |(None, ((n,None),c), _) -> {acc with install = ((n,Some arch),c) :: acc.install}
+        |(Some Format822.I, ((n,None),c), _) -> {acc with install = ((n,arch),c) :: acc.install}
+        |(Some Format822.R, ((n,None),c), _) -> {acc with remove =  ((n,arch),c) :: acc.remove}
+        |(None, ((n,None),c), _) -> {acc with install = ((n,arch),c) :: acc.install}
 
         |(Some Format822.I, vpkg, _) -> {acc with install = vpkg :: acc.install}
         |(Some Format822.R, vpkg, _) -> {acc with remove = vpkg :: acc.remove}
@@ -53,9 +60,9 @@ let from_apt_request arch request = function
       ) request vpkgreqlist
   |Apt.Remove vpkgreqlist ->
       List.fold_left (fun acc -> function
-        |(Some Format822.I, ((n,None),c), _) -> {acc with install = ((n,Some arch),c) :: acc.install}
-        |(Some Format822.R, ((n,None),c), _) -> {acc with remove = ((n,Some arch),c) :: acc.remove}
-        |(None, ((n,None),c), _) -> {acc with remove = ((n,Some arch),c) :: acc.remove}
+        |(Some Format822.I, ((n,None),c), _) -> {acc with install = ((n,arch),c) :: acc.install}
+        |(Some Format822.R, ((n,None),c), _) -> {acc with remove = ((n,arch),c) :: acc.remove}
+        |(None, ((n,None),c), _) -> {acc with remove = ((n,arch),c) :: acc.remove}
 
         |(Some Format822.I, vpkg, _) -> {acc with install = vpkg :: acc.install}
         |(Some Format822.R, vpkg, _) -> {acc with remove = vpkg :: acc.remove}
@@ -67,22 +74,60 @@ let from_apt_request arch request = function
 
 let parse_s = Packages.parse_s
 let parse_string (_,s) = s
+let parse_string_opt = function (_,"") -> None | (_,s) -> Some s
+let parse_string_list (_,s) = String.nsplit s " "
 let parse_int_s (_,s) = string_of_int(int_of_string s)
 let parse_req (loc,s) = 
   let aux = Packages.lexbuf_wrapper Packages_parser.vpkg_top in
   let l = Pcre.split ~rex:Apt.blank_regexp s in 
   List.map (fun s -> aux (loc,s)) l
 
+let parse_edsp_version (_,s) =
+  match String.nsplit s " " with
+  |["EDSP";s] when (float_of_string s) >= 0.4 -> s
+  |_ -> raise Not_found
+
+(*
+let get_architectures native_opt foreign =
+  let cmd = "apt-config dump" in
+  let arch = ref "" in
+  let archs = ref [] in
+  let aux () =
+    let out = Std.input_list (Unix.open_process_in cmd) in
+    List.iter (fun s ->
+      let key, value =  ExtString.String.split s " " in
+      if key = "APT::Architecture" then
+        arch := ExtString.String.slice ~first: 1 ~last:(-2) value
+      else if key = "APT::Architectures::" || key = "APT::Architectures" then
+        let s = ExtString.String.slice ~first:1 ~last:(-2) value in
+        if s <> "" then
+          archs := (ExtString.String.slice ~first:1 ~last:(-2) value)::!archs
+    ) out;
+    debug "Atomatically set native as %s and foreign archs as %s" !arch (String.concat "," !archs);
+  in
+  match native_opt, foreign with 
+  |None,None     -> aux () ; (!arch,List.filter ((<>) !arch) !archs)
+  |None,Some l   -> fatal "Native arch is missing while Foregin archs are specified"
+  |Some a,None   -> (a,[])
+  |Some a,Some l -> (a,l)
+;;
+*)
+
 let parse_request_stanza par =
+  (* request must be parse before any other fields *)
+  let request = parse_s ~err:"(Invalid EDSP version)" parse_edsp_version "Request" par in
   {
-    request = parse_s ~err:"(Empty REQUEST)" parse_string "Request" par;
+    request = request; 
     install = parse_s ~opt:[] parse_req "Install" par;
     remove = parse_s ~opt:[] parse_req "Remove" par;
     upgrade = parse_s ~opt:false Packages.parse_bool "Upgrade" par;
+    architecture = parse_s ~opt:None parse_string_opt "Architecture" par;
+    architectures = parse_s ~opt:[] parse_string_list "Architectures" par;
     distupgrade = parse_s ~opt:false Packages.parse_bool "Dist-Upgrade" par;
     autoremove = parse_s ~opt:false Packages.parse_bool "Autoremove" par;
     strict_pin = parse_s ~opt:true Packages.parse_bool "Strict-Pinning" par;
     preferences = parse_s ~opt:"" Packages.parse_string "Preferences" par;
+    cmdline = parse_s ~opt:"" Packages.parse_string "Command-Line" par;
   }
 ;;
 
@@ -109,10 +154,13 @@ let extras = [
   ("APT-Candidate", Some parse_candidate);
   ("APT-Automatic", Some parse_automatic);
   ("Section", Some parse_section);
+  ("APT-Release", None);
   ]
 
 (* parse the entire file while filtering out unwanted stanzas *)
-let rec packages_parser ?(request=false) archs (req,acc) p =
+let rec packages_parser ?(request=false) (req,acc) p =
+  (* if strict pin = true we filter packages that are installed and
+     apt-candidate *)
   let filter par = 
     let match_field f p =
       try 
@@ -123,41 +171,45 @@ let rec packages_parser ?(request=false) archs (req,acc) p =
         end
       with Not_found -> false
     in
-    let inst () = match_field "installed" par in 
-    let candidate () = match_field "apt-candidate" par in
+    let inst () = match_field "Installed" par in 
+    let candidate () = match_field "APT-Candidate" par in
     ((inst ()) || (candidate ()))
   in
+  let archs =
+    if Option.is_none req.architecture then []
+    else (Option.get req.architecture)::req.architectures 
+  in
   match Format822_parser.stanza_822 Format822_lexer.token_822 p.Format822.lexbuf with
   |None -> (req,acc) (* end of file *)
   |Some stanza when request = true -> 
       let req = parse_request_stanza stanza in
-      packages_parser archs (req,acc) p
+      packages_parser (req,acc) p
   |Some stanza when req.strict_pin = true -> begin
     match (Packages.parse_package_stanza (Some(filter)) archs extras stanza) with
-    |None -> packages_parser archs (req,acc) p
-    |Some st -> packages_parser archs (req,st::acc) p
+    |None -> packages_parser (req,acc) p
+    |Some st -> packages_parser (req,st::acc) p
   end
   |Some stanza when req.strict_pin = false -> begin
     match (Packages.parse_package_stanza None archs extras stanza) with
     |None -> assert false (* this is not possible in this branch *)
-    |Some st -> packages_parser archs (req,st::acc) p
+    |Some st -> packages_parser (req,st::acc) p
   end
   |_ -> assert false
 ;;
 
-let input_raw_ch ?(archs=[]) ic =
+let input_raw_ch ic =
   Format822.parse_from_ch (
-    packages_parser ~request:true archs (default_request,[])
+    packages_parser ~request:true (default_request,[])
   ) ic
 ;;
 
-let input_raw ?(archs=[]) file =
+let input_raw file =
   let ch =
     match file with
     |"-" -> IO.input_channel stdin
     |_   -> Input.open_file file
   in
-  let l = input_raw_ch ~archs ch in
+  let l = input_raw_ch ch in
   let _ = Input.close_ch ch in
   l
 ;;
@@ -176,14 +228,14 @@ let extras_tocudf =
 let is_installed pkg =
   try
     let _loc = Format822.dummy_loc in
-    let v = Packages.assoc "installed" pkg.Packages.extras in
+    let v = Packages.assoc "Installed" pkg.Packages.extras in
     Packages.parse_bool (_loc,v)
   with Not_found -> false
 
 let is_on_hold pkg =
   try
     let _loc = Format822.dummy_loc in
-    let v = Packages.assoc "hold" pkg.Packages.extras in
+    let v = Packages.assoc "Hold" pkg.Packages.extras in
     (Packages.parse_bool (_loc,v))
   with Not_found -> false
 
@@ -195,10 +247,10 @@ let tocudf tables ?(options=Debcudf.default_options) ?(inst=false) pkg =
         if is_on_hold pkg then "hold ok installed" 
         else "install ok installed"
       in
-      { pkg with Packages.extras = ("status",s)::pkg.Packages.extras }
+      { pkg with Packages.extras = ("Status",s)::pkg.Packages.extras }
     else pkg
   in
-  Debcudf.tocudf tables ~options (* ~inst:(is_installed pkg) *) pkg
+  Debcudf.tocudf tables ~options pkg
 ;;
 
 (* Only one version of a package can be installed at a given time.
