@@ -1,14 +1,19 @@
-(**************************************************************************************)
-(*  Copyright (C) 2009 Pietro Abate <pietro.abate@pps.jussieu.fr>                     *)
-(*  Copyright (C) 2009 Mancoosi Project                                               *)
-(*                                                                                    *)
-(*  This library is free software: you can redistribute it and/or modify              *)
-(*  it under the terms of the GNU Lesser General Public License as                    *)
-(*  published by the Free Software Foundation, either version 3 of the                *)
-(*  License, or (at your option) any later version.  A special linking                *)
-(*  exception to the GNU Lesser General Public License applies to this                *)
-(*  library, see the COPYING file for more information.                               *)
-(**************************************************************************************)
+(******************************************************************************)
+(*  This file is part of the Dose library http://www.irill.org/software/dose  *)
+(*                                                                            *)
+(*  Copyright (C) 2009-2011 Pietro Abate <pietro.abate@pps.jussieu.fr>        *)
+(*                                                                            *)
+(*  This library is free software: you can redistribute it and/or modify      *)
+(*  it under the terms of the GNU Lesser General Public License as            *)
+(*  published by the Free Software Foundation, either version 3 of the        *)
+(*  License, or (at your option) any later version.  A special linking        *)
+(*  exception to the GNU Lesser General Public License applies to this        *)
+(*  library, see the COPYING file for more information.                       *)
+(*                                                                            *)
+(*  Work developed with the support of the Mancoosi Project                   *)
+(*  http://www.mancoosi.org                                                   *)
+(*                                                                            *)
+(******************************************************************************)
 
 (** Representation of a debian package description item. *)
 
@@ -25,6 +30,7 @@ type package = {
   multiarch : Format822.multiarch ;
   essential : bool;
   build_essential : bool;
+  extra_source_only : bool;
   priority : string;
   source : (Format822.name * Format822.version option) ;
   depends : Format822.vpkgformula ;
@@ -45,6 +51,7 @@ let default_package = {
   architecture = "";
   multiarch = `None;
   essential = false;
+  extra_source_only = false;
   build_essential = false;
   priority = "";
   depends = [];
@@ -79,10 +86,9 @@ let parse_binarylist = lexbuf_wrapper Packages_parser.vpkglist_top
 
 (* assume n is lowercase *)
 let rec assoc (n : string) = function
-  |(k,v)::_ when (String.lowercase k) = n -> v
+  |(k,v)::_ when k = n -> v
   |(k,_)::t -> assoc n t
   |[] -> raise Not_found
-;;
 
 exception ParseError of string * string
 exception IgnorePackage of string
@@ -91,12 +97,12 @@ exception IgnorePackage of string
  * opt = None && err = Some s -> ParseError s :
  * opt = Some s -> return s *)
 let parse_s ?opt ?err ?(multi=false) f field par =
-  let field = String.lowercase field in
   try let (_loc,s) = (assoc field par) in f (_loc,s) 
   with Not_found ->
     if Option.is_none opt then
       if Option.is_none err then raise Not_found
       else begin
+        (*
         let (_,((startpos,endpos),_)) = List.hd par in
         let s = 
           Printf.sprintf "%s : %s--%s" 
@@ -104,6 +110,8 @@ let parse_s ?opt ?err ?(multi=false) f field par =
           (Format822.pp_lpos startpos) 
           (Format822.pp_lpos endpos)
         in
+        *)
+        let s = "" in
         raise (ParseError (field,(Option.get err)^" (no default declared) " ^ s))
       end
     else Option.get opt
@@ -153,6 +161,7 @@ let parse_package_stanza filter archs extras par =
       source = parse_s ~opt:("",None) parse_source "Source" par;
 
       essential = parse_s ~opt:false parse_bool "Essential" par;
+      extra_source_only = parse_s ~opt:false parse_bool "Extra-Source-Only" par;
       build_essential = parse_s ~opt:false parse_bool "Build-Essential" par;
       priority = parse_s ~opt:"" parse_string "Priority" par;
 
@@ -191,7 +200,7 @@ let parse_package_stanza filter archs extras par =
 
 let status_filter par =
   try
-    let (_,s) = (assoc "status" par) in
+    let (_,s) = (assoc "Status" par) in
     match String.nsplit s " " with
     |[_;_;"installed"] -> true
     |_ -> false
@@ -199,7 +208,7 @@ let status_filter par =
 
 let arch_filter archlist par =
   try
-    let (_,s) = (assoc "architecture" par) in
+    let (_,s) = (assoc "Architecture" par) in
     List.mem s archlist
   with Not_found -> false
 
@@ -209,9 +218,9 @@ let rec packages_parser fname stanza_parser acc p =
   match Format822_parser.stanza_822 Format822_lexer.token_822 p.Format822.lexbuf with
   |None -> acc
   |Some stanza -> begin
-      match stanza_parser (filename::stanza) with
-      |None -> packages_parser fname stanza_parser acc p
-      |Some st -> packages_parser fname stanza_parser (st::acc) p
+    match stanza_parser (filename::stanza) with
+    |None -> packages_parser fname stanza_parser acc p
+    |Some st -> packages_parser fname stanza_parser (st::acc) p
   end
 
 let parse_packages_in ?filter ?(archs=[]) ?(extras=[]) fname ic =
@@ -229,8 +238,9 @@ module Set = struct
     type t = package
     let compare x y =
       let c = x >% y in 
-      if c = 0 then 
-        debug "the input contains two packages with the same name, version and architecture (%s,%s,%s). Only the latter will be considered."
+      if c = 0 && x.architecture <> "all" then
+        debug
+        "the input contains two packages with the same name, version and architecture (%s,%s,%s). Only the latter will be considered."
         x.name x.version x.architecture;
       c
   end)
@@ -238,34 +248,44 @@ end
 (**/**)
 
 let merge status packages =
-  info "Merging status file";
-  let merge_aux p1 p2 =
-    if (p1 >% p2) = 0 then begin
-      {p1 with
-        essential = p1.essential || p2.essential;
-        extras = List.unique (p1.extras @ p2.extras)
-      }
-    end else fatal "Something went wrong while merging status+packages"
-  in
-  let h = Hashtbl.create (List.length status) in
-  List.iter (fun p -> Hashtbl.add h (id p) p) status ;
-  let ps =
-    List.fold_left (fun acc p ->
-      try Set.add (merge_aux p (Hashtbl.find h (id p))) acc
-      with Not_found -> Set.add p acc
-    ) Set.empty (status @ packages)
-  in
-  Set.elements ps 
+  if List.length status > 0 then begin
+    info "Merging status file";
+    let merge_aux p1 p2 =
+      if (p1 >% p2) = 0 then begin
+        {p1 with
+          essential = p1.essential || p2.essential;
+          extras = List.unique (p1.extras @ p2.extras)
+        }
+      end else fatal "Something went wrong while merging status+packages"
+    in
+    let h = Hashtbl.create (List.length status) in
+    List.iter (fun p -> Hashtbl.add h (id p) p) status ;
+    let ps =
+      List.fold_left (fun acc p ->
+        try Set.add (merge_aux p (Hashtbl.find h (id p))) acc
+        with Not_found -> Set.add p acc
+      ) Set.empty (status @ packages)
+    in
+    Set.elements ps 
+  end
+  else 
+    packages
+;;
 
-let is_installed pkg =
-  try match String.nsplit (assoc "status" pkg.extras) " " with
+let installed_re = Re_pcre.regexp "[a-z]+[ \t]+[a-z]+[ \t]+installed"
+let is_installed pkg = 
+  try Re_pcre.pmatch ~rex:installed_re (assoc "Status" pkg.extras)
+  with Not_found -> false
+  (*
+  try match String.nsplit (assoc_string "Status" pkg.extras) " " with
     |[_;_;"installed"] -> true
     | _ -> false
   with Not_found -> false
+  *)
 
 let is_on_hold pkg =
-  try match String.nsplit (assoc "status" pkg.extras) " " with
-    |["hold";_;_] -> true
+  try match String.split (assoc "Status" pkg.extras) " " with
+    |"hold",_ -> true
     | _ -> false
   with Not_found -> false
 
@@ -273,6 +293,7 @@ let default_extras = [
   ("Status", None);
   ("Size", None);
   ("Installed-Size", None);
+  ("Multi-Arch", None);
   ("Filename", None);
 ]
 
