@@ -138,7 +138,11 @@ let architecture_test_cases = [
   ("linux-any", "netbsd-alpha", false);
   ("any-any", "linux-i386", true);     (* OS and CPU pattern *)
   ("any-any", "hurd-i386", true);
-  ("any-any", "amd64", true)
+  ("any-any", "amd64", true);
+  ("any-arm", "armhf", true);          (* arch name is not equal cpu name *)
+  ("any-i386", "lpia", true);
+  ("any-amd64", "x32", true);
+  ("any-powerpc", "powerpcspe", true)
 ];;
 
 let test_architecture_matching =
@@ -236,40 +240,77 @@ let rec assert_delay_stub l =
     end
 ;;
 
+module PkgSetTest = OUnitDiff.SetMake (struct
+  type t = (string * string)
+  let compare = compare
+  let pp_printer ppf (p,v) = Format.fprintf ppf "(%s,%s)" p v
+  let pp_print_sep = OUnitDiff.pp_comma_separator
+end)
+
+module SubClusterSetTest = OUnitDiff.SetMake (struct
+  type t = (string * string * PkgSetTest.t)
+  let compare = compare
+  let pp_printer ppf (p,v,s) = Format.fprintf ppf "(%s,%s,{%a})" p v PkgSetTest.pp_printer s
+  let pp_print_sep = OUnitDiff.pp_comma_separator
+end)
+
+module ClusterSetTest = OUnitDiff.SetMake (struct
+  type t = (string * string * SubClusterSetTest.t)
+  let compare = compare
+  let pp_printer ppf (p,v,s) = Format.fprintf ppf "(%s,%s,{%a})" p v SubClusterSetTest.pp_printer s
+  let pp_print_sep = OUnitDiff.pp_comma_separator
+end)
+
+
 let test_cluster =
   let packagelist = Packages.input_raw [f_discriminants] in
   let clusters = Debutil.cluster packagelist in
   "cluster" >::: [
     "groups" >:: (fun _ -> 
-      let assert_delay = 
-        assert_delay_stub [
-          ("bb","1",[("bb","1")]);
-          ("aa","1",[("aa","1")]);
-          ("ee_source","2",[("ff","2")]);
-          ("ee_source","1",[("gg","1");("ee","1")]);
-          ("cc_source","1",[("cc","2")]);
-          ("cc_source","1",[("dd","1")]);
+      let of_list ll = 
+        ClusterSetTest.of_list (
+          List.map (fun (p,v,l) -> (p,v,
+            SubClusterSetTest.of_list (
+              List.map (fun (v,rv,cl) ->
+                (v,rv,PkgSetTest.of_list cl)
+              ) l
+            ))
+          ) ll 
+        )
+      in
+      let expected = of_list [
+        (* sourcename, sourceversion, [ list of clusters
+         * cluster version (normalized), real version, [list of packages in the cluster]
+         * ] 
+         *)
+        ("bb","1",["1","1",[("bb","1")]]);
+        ("aa","1",["1","1",[("aa","1")]]);
+        ("ee_source","1",[
+          "1","1",[("gg","1");("ee","1")]]);
+        ("ee_source","2",["2","2",[("ff","2")]]);
+        ("cc_source","1",[
+          "1","1",[("dd","1")];
+          "2","2",[("cc","2")]
+          ]);
         ]
       in
-
-      Hashtbl.iter (fun (sourcename, sourceversion) l ->
-        (* Printf.eprintf "(1)cluster (%s,%s)\n%!" sourcename sourceversion; *)
-        List.iter (fun (version,realversion,cluster) ->
-          (* Printf.eprintf "(1)v %s\n%!" version; *)
-          let l = List.map(fun pkg -> (pkg.Packages.name,pkg.Packages.version)) cluster in
-          (*
-          List.iter (fun pkg ->
-            Printf.eprintf "(1)pkg %s %s\n%!" pkg.Packages.name pkg.Packages.version
-          ) cluster;
-          *)
-          assert_delay (sourcename,sourceversion,l);
-        ) l
-      ) clusters
+      let result =
+        ClusterSetTest.of_list (
+          Hashtbl.fold (fun (sn, sv) l acc ->
+            (sn,sv,SubClusterSetTest.of_list (
+              List.map (fun (v,rv,cluster) -> 
+                let cl = List.map(fun pkg -> (pkg.Packages.name,pkg.Packages.version)) cluster in
+                (v,rv,PkgSetTest.of_list cl)
+              ) l
+            ))::acc
+          ) clusters []
+        )
+      in
+      ClusterSetTest.assert_equal expected result
     ); 
   ]
 ;;
 
-
 let test_evolution =
   let packagelist = Packages.input_raw [f_discriminants] in
   let constraints_table = Evolution.constraints packagelist in
@@ -494,8 +535,8 @@ let test_mapping =
 
 (* Useful test functions *)
 
-let returns_result function_to_test expected_result =
-  (fun args () -> assert_equal (function_to_test args) expected_result)
+let returns_result ?(printer=(fun _ -> "(FIXME)")) function_to_test expected_result =
+  (fun args () -> assert_equal ~printer (function_to_test args) expected_result)
 and raises_failure function_to_test failure_text =
   (fun args () -> assert_raises (Failure failure_text) (fun () -> function_to_test args) )
 
@@ -569,7 +610,7 @@ let parse_inst_triplets =
   ]
 
 let list_of_hashtbl ht = 
-  ExtLib.List.of_enum (ExtLib.Hashtbl.enum ht)
+  List.sort compare (ExtLib.List.of_enum (ExtLib.Hashtbl.enum ht))
 
 let test_parse_inst = 
   let parse_inst_test_cases =
@@ -754,17 +795,19 @@ Build-Depends-Indep: bin3
 let test_sources2packages =
   let data = IO.input_string test_sources_input in
   let packagelist = Sources.parse_sources_in "" data in
-  let builddeparch = "amd64" in
-  let sources = Sources.sources2packages ~profiles:true builddeparch packagelist in
+  let hostarch = "amd64" in
+  let buildarch = "amd64" in
+  let sources = Sources.sources2packages ~profiles:true buildarch hostarch packagelist in
   let function_to_test src =
     let src = List.find (fun s -> s.Packages.name = src) sources in
     src.Packages.depends
   in
-  let returns = returns_result function_to_test in
+  let printer = Printer.string_of_vpkgformula in
+  let returns = returns_result ~printer function_to_test in
   [
     (
       "any/native", "src:source1", returns [
-        [(("build-essential", Some "native"), None)];
+        [(("build-essential", Some hostarch), None)];
         [(("bin1", None), None)];
         [(("bin2", Some "any"), None)];
         [(("bin3", Some "native"), None)]
@@ -772,14 +815,14 @@ let test_sources2packages =
     );
     (
       "default", "src:source2", returns [
-        [(("build-essential", Some "native"), None)];
+        [(("build-essential", Some hostarch), None)];
         [(("bin1", None), None)];
         [(("bin2", None), None)]
       ]
     );
     (
       "stage1", "src-stage1:source2", returns [
-        [(("build-essential", Some "native"), None)];
+        [(("build-essential", Some hostarch), None)];
         [
           (("bin2", None), None);
           (("bin3", None), None)
@@ -788,7 +831,7 @@ let test_sources2packages =
     );
     (
       "indep", "src:source3", returns [
-        [(("build-essential", Some "native"), None)];
+        [(("build-essential", Some hostarch), None)];
         [(("bin3", Some "native"), None)];
         [(("bin1",None), None)];
         [(("bin2",None), None)]
